import os
import time
import subprocess
import time
import board
import RPi.GPIO as GPIO
from datetime import datetime
from datetime import date
from os.path import exists
from adafruit_lc709203f import LC709203F

pcm = True
threshold = 3.7

logPath = '/home/pi/src/python/log/'

volts = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
suns = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
samplingInterval = 2
meanInterval = 30
length = meanInterval / samplingInterval

def logDay():
    day =  date.today()
    day = day.strftime("%d_%m_%Y")    
    return day

def logTime():
    now = datetime.now()
    current_time = now.strftime("%H:%M:%S")    
    return current_time

def createLog():
    with open(logPath+logDay()+'.txt', 'w') as f:
        f.write("TIME")
        f.write('\t')
        f.write('\t')
        f.write("VOLT")
        f.write('\t')
        f.write("SUN")
        f.write('\t')
        f.write('\t')
        f.write("STATE")
        f.write('\n')

def updateLog():
    if exists(logPath+logDay()+'.txt'):
        with open(logPath+logDay()+'.txt', "a") as f:
            f.write("-----------------------------------------------\n")
    else:
        print("Generating log")
        createLog()
        with open(logPath+logDay()+'.txt', "a") as f:
            f.write("-----------------------------------------------\n")
    if pcm:
        with open(logPath+logDay()+'.txt', "a") as f:
            f.write('System booted wiht PCM ON (shutdown is True)\n')
            f.write("-----------------------------------------------\n")
            f.write("TIME")
            f.write('\t')
            f.write('\t')
            f.write("VOLT")
            f.write('\t')
            f.write("SUN")
            f.write('\t')
            f.write('\t')
            f.write("STATE")
            f.write('\n')
    else:
        with open(logPath+logDay()+'.txt', "a") as f:
            f.write('System booted wiht PCM OFF (shutdown is False)\n')
            f.write("-----------------------------------------------\n")
            f.write("TIME")
            f.write('\t')
            f.write('\t')
            f.write("VOLT")
            f.write('\t')
            f.write("SUN")
            f.write('\t')
            f.write('\t')
            f.write("STATE")
            f.write('\n')
            
def append(volt, sun, state):
    with open(logPath+logDay()+'.txt', "a") as f:
        f.write(logTime())
        f.write('\t')
        f.write(str(format(volt, '.3f')))
        f.write('\t')
        f.write(str(sun))
        f.write('\t')
        f.write(state)
        f.write('\n')

def log(volt, sun, state):
    if exists(logPath+logDay()+'.txt'):
        append(volt, sun, state)
    else:
        print("Generating log")
        createLog()
        append(volt, sun, state)

GPIO.setmode(GPIO.BCM)
sunPin = 13
GPIO.setup(sunPin, GPIO.IN)

sensor = LC709203F(board.I2C())
sensor._write_word(0x14, int(threshold*1000)) 

def shutdown():
    print("60 seconds to shutdown")
    time.sleep(60)
    print("shutdown")
    os.system("sudo shutdown -h now")

def average(lst):
    return sum(lst) / len(lst)

def check(volt, sun):
    if sun:
        if volt > (threshold - 0.05):
            log(volt, sun, "P")
        else:
            log(volt, sun, "S")
            if pcm:
                shutdown()
    else:
        if volt > (threshold + 0.05):
            log(volt, sun, "S")
            if pcm:
                shutdown()
        else:
            if volt > (threshold - 0.05):
                log(volt, sun, "W")
            else:
                log(volt, sun, "S")
                if pcm:
                    shutdown()

def loop():
    sampleIndex = 0
    while True:
        sun = not GPIO.input(sunPin)
        try:
            volt = sensor.cell_voltage
            volts[sampleIndex] = volt
            suns[sampleIndex] = sun
            sampleIndex+=1
            print("volt: " + str(volt) + " sun: " + str(sun))
            if sampleIndex == length:
                check(average(volts), suns.count(True) > 0)
                print("volt: " + str(format(average(volts), '.3f')) + " sun: " + str(suns.count(True) > 0) + " (mean)")
                sampleIndex = 0
        except:
            #log(6.666, sun, "FAIL") #skip when reading error
            print("RuntimeError: CRC failure on reading word")
            pass
        time.sleep(samplingInterval)
        
def init():
    GPIO.setup(sunPin, GPIO.IN)
    if GPIO.input(sunPin):
        print("no sun")
    else:
        print("sun")
    
def main():
    print("!!!!!!!!!!!!!!INIT")
    updateLog()
    init()
    time.sleep(1)
    print("!!!!!!!!!!!!!!LOOP")
    loop()
    
main()

